/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Enemy class

#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <GL/gl.h>
#include "Enemy.h"
#include "MyTrig.h"
#include "Projectile.h"
#include "main.h"

void Enemy::Draw() {
		glPushMatrix();
		glColor3f(0.9,0.9,0.9);
		glTranslatef(x, y, z);
		glRotatef(270-rotation, 0, 1, 0);
		glRotatef(-90, 1, 0, 0);
		glCallList(model_airplane);
		glPopMatrix();
}

Enemy::Enemy() {
	// Place enemy on a random edge of the map (300x300 meters)
	int edge = rand()%4;
	switch(edge) {
		case 0:
			x = rand()%401-200;
			z = -200;
			break;
		
		case 1:
			x = rand()%401-200;
			z = 200;
			break;
		
		case 2:
			x = -200;
			z = rand()%401-200;
			break;
		
		case 3:
			x = 200;
			z = rand()%401-200;
			break;
	}
	
	// In the easy game, planes go in random directions
	// In the hard game, they head straight for home 50% of the time
	if(!easygame && rand() < RAND_MAX/4) {
		rotation = atan2(-z, -x)*TODEG;
		target_x = target_z = 0.0;
	}
	else {
		if(alive_building.size() <= 0) {
			// Head within a 120 degree range of the center
			rotation = atan2(-z, -x)*TODEG - 60 + rand()%12000/100.0;
		}
		else {
			// Pick a random alive building and head for it
			int bldg = rand()%alive_building.size();
			list<Building*>::iterator it = alive_building.begin();
			int i;
			for(i=0; i<bldg; i++) ++it;
			Building *b = *it;
			rotation = atan2(z-b->y, x-b->x)*TODEG-180;
			target_x = b->x + 5;
			target_z = b->y + 5;
		}
	}

	y = 65 + rand()%7500/100.0;
	velocity = 10 + rand()%2500/100.0;
	bomb_dropped = false;
}

bool Enemy::Update(float dt) {
	x += velocity * cos(rotation*TORAD) * dt;
	z += velocity * sin(rotation*TORAD) * dt;
	
	if(!bomb_dropped && fabs(x-target_x)<=4.5 && fabs(z-target_z)<=4.5) {
		// Drop the bomb
		Projectile *p = new Projectile(x, y, z, rotation, -90, 10);
		projectile.push_back(p);
		bomb_dropped = true;
	}
	
	if(fabs(x) > 200 || fabs(z) > 200) return false;
	return true;
}

bool Enemy::isInside(float x, float y, float z) {
	if(x >= this->x+plane_minx
	&& x <= this->x+plane_maxx
	&& y >= this->y+plane_miny
	&& y <= this->y+plane_maxy
	&& z >= this->z+plane_minz
	&& z <= this->z+plane_maxz)
		return true;
	return false;
}
