/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Explosion class

#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>
#include "MyTrig.h"
#include "main.h"
#include "gfx.h"
#include "Explosion.h"

Explosion::Explosion(float x, float y, float z, int type) {
	this->x = x;
	this->y = y;
	this->z = z;
	angle = rand()%36000/100.0;
	
	
	switch(type) {
	case EX_TYPE_FIRE:
		frame = FIRE00 + rand()%8;
		dx = -12.5 + rand()%25000/1000.0;
		dz = -12.5 + rand()%25000/1000.0;
		dy = -4.167 + rand()%25000/3500.0;
		ttl = 5 + rand()%10000/1000.0;
		rotate_speed = -120 + rand()%24000/100.0;
		decay_rate = 1.0;//1.5 + rand()%1500/1000.0;
		break;
	
	case EX_TYPE_SMOKE:
		frame = SMOKE00 + rand()%8;
		dx = dz = 0;
		dy = 1.0;
		ttl = 5 + rand()%10000/1000.0;
		rotate_speed = -75 + rand()%15000/100.0;
		decay_rate = 0.75 + rand()%5000/10000.0;
		break;
	
	case EX_TYPE_PARTICLE:
		break;
	}
}


bool Explosion::Update(float dt) {
	ttl -= dt*decay_rate;
	if(ttl <= 0.0) return false;
	
	if(frame < SMOKE00) dy -= 5*dt;
	
	angle += rotate_speed*dt;
	x += dx*dt;
	y += dy*dt;
	z += dz*dt;
	
	if(y < 0.0) return false;
	
	return true;
}


void Explosion::Draw() {
	glPushMatrix();
	glTranslatef(x, y, z);
	glRotatef(180-camera.rotation, 0, 1, 0);
	glRotatef(180-camera.tilt, 0, 0, 1);
	glRotatef(angle, 1, 0, 0);
	

	glDepthMask(GL_FALSE);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glBindTexture(GL_TEXTURE_2D, texture[frame]);
	glBegin(GL_QUADS);
		glColor4f(1, 1, 1, ttl/15.0 * (frame<SMOKE00?1.0:0.7));
		glTexCoord2i(0,0);	glVertex3f(0, -2, -2);
		glTexCoord2i(0,1);	glVertex3f(0, -2, 2);
		glTexCoord2i(1,1);	glVertex3f(0, 2, 2);
		glTexCoord2i(1,0);	glVertex3f(0, 2, -2);
	glEnd();
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glDepthMask(GL_TRUE);
	glEnable(GL_LIGHTING);
	glPopMatrix();
}
