/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Model loading

#include <math.h>
#include <GL/gl.h>
#include <alleggl.h>
#include <stdio.h>
#include "Model.h"

#define MIN3(a, b, c)	( MIN(a, MIN(b, c)) )
#define MAX3(a, b, c)	( MAX(a, MAX(b, c)) )
#define MIN4(a, b, c, d)	( MIN3(a, b, MIN(c, d)) )
#define MAX4(a, b, c, d)	( MAX3(a, b, MAX(c, d)) )

GLuint LoadModel(const char *filename, float *minx, float *maxx, float *miny, float *maxy, float *minz, float *maxz) {
	fprintf(stderr, "Reading %s...", filename);

	// Temporary variables for the largest and smallest x, y, z values
	float minxx, minyy, minzz, maxxx, maxyy, maxzz;
	bool first=true;
	
	FILE *f = fopen(filename, "rt");
	if(!f) return 0;
	
	glEnable(GL_NORMALIZE);
	
	GLuint ret = glGenLists(1);
	glNewList(ret, GL_COMPILE);
	glBegin(GL_TRIANGLES);
	
		// Read the first line: it is just a comment
		char line[512];
		fgets(line, 512, f);
	
		// Read every other line, they each contain three vertices in three dimensions
		int tris=0;
		while(!feof(f)) {
			float v[9];
			if(fscanf(f, "%f %f %f %f %f %f %f %f %f\n", &v[0], &v[1], &v[2], &v[3], &v[4], &v[5], &v[6], &v[7], &v[8]) == 9) {
				tris++;
				// Compute the normal
				// NOTE: I don't think this is correct, check it please!
				float n1x = v[0]-v[3];
				float n1y = v[1]-v[4];
				float n1z = v[2]-v[5];
				
				float n2x = v[3]-v[6];
				float n2y = v[4]-v[7];
				float n2z = v[5]-v[8];
			
				glNormal3f(n1y*n2z - n2y*n1z, n1z*n2x- n2z*n1x, n1x*n2y - n1y*n2x);
			
				// Assign the vertices
				glVertex3f(v[0], v[1], v[2]);
				glVertex3f(v[3], v[4], v[5]);
				glVertex3f(v[6], v[7], v[8]);
			
				// Check for min/max x/y/z
				if(first) {
					first = false;
					minxx = MIN3(v[0], v[3], v[6]);
					maxxx = MAX3(v[0], v[3], v[6]);
					minyy = MIN3(v[1], v[4], v[7]);
					maxyy = MAX3(v[1], v[4], v[7]);
					minzz = MIN3(v[2], v[5], v[8]);
					maxzz = MAX3(v[2], v[5], v[8]);
				}
				else {
					minxx = MIN4(v[0], v[3], v[6], minxx);
					maxxx = MAX4(v[0], v[3], v[6], maxxx);
					minyy = MIN4(v[1], v[4], v[7], minyy);
					maxyy = MAX4(v[1], v[4], v[7], maxyy);
					minzz = MIN4(v[2], v[5], v[8], minzz);
					maxzz = MAX4(v[2], v[5], v[8], maxzz);
				}
			}
			else fprintf(stderr, "Error scanning 9 objects!\n");
		}
	
	glEnd();
	glEndList();
	glDisable(GL_NORMALIZE);
	
	fclose(f);
	fprintf(stderr, "...%d triangles read as display list %d.\n", tris, ret);
		
	// Store the min/max x/y/z values
	*minx = minxx;
	*maxx = maxxx;
	*miny = minyy;
	*maxy = maxyy;
	*minz = minzz;
	*maxz = maxzz;
	fprintf(stderr, "min: (%f, %f, %f);\tmax: (%f, %f, %f)\n", *minx, *miny, *minz, *maxx, *maxy, *maxz);

	return ret;
}
