/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Projectile class

#include <allegro.h>
#include <GL/gl.h>
#include <math.h>
#include "Projectile.h"
#include "Explosion.h"
#include "MyTrig.h"
#include "sfx.h"
#include "main.h"

Projectile::Projectile(float x, float y, float z, float rot, float til, float vel) {
	this->x = x;
	start_y = this->y = y;
	this->z = z;
	
	rotation = rot;
	tilt = til;
	velocity = vel;
	smoke_timer=0.1;
	
	if(tilt == -90) {
		// If this projectile was dropped from a plane, try to give it a voice
		voice = allocate_voice((SAMPLE*)sfx[SFX_WHISTLE].dat);
		if(voice == -1) return;
		if(voice > 4) {
			deallocate_voice(voice);
			voice = -1;
			return;
		}
		
		voice_start(voice);
		voice_set_frequency(voice, int(y/100.0*44100));
		float dist = sqrt(x*x+(y-camera.z)*(y-camera.z)+z*z);
		voice_set_volume(voice, int(64*dist/150));
		voice_set_playmode(voice, PLAYMODE_LOOP);
	}
	else voice = -1;
}

Projectile::~Projectile() {
	if(voice != -1) {
		voice_stop(voice);
		deallocate_voice(voice);
	}
}

bool Projectile::Update(float dt) {
	x += velocity * cos(tilt*TORAD) * cos(rotation*TORAD) * dt;
	y += velocity * sin(tilt*TORAD) * dt;
	z += velocity * cos(tilt*TORAD) * sin(rotation*TORAD) * dt;
	
	smoke_timer -= dt;
	if(smoke_timer <= 0.0) {
		smoke_timer = 0.2;
		Explosion *e = new Explosion(x, y, z, EX_TYPE_SMOKE);
		explosion.push_back(e);
	}
	
	// Update sample, if applicable
	if(voice != -1) {
		voice_set_frequency(voice, int(y/100.0*44100));
		float dist = sqrt(x*x+(y-camera.z)*(y-camera.z)+z*z);
		voice_set_volume(voice, int(64*dist/150));
	}
	
	// If we're out of bounds, return false, so we know to delete this item
	if(fabs(x) > 200 || fabs(z) > 200 || y < 0 || y > 200) return false;	
	return true;
}


void Projectile::Draw() {
	glPushMatrix();
	glColor3ub(255,186,39);
	glTranslatef(x, y, z);
	glRotatef(270-rotation, 0, 1, 0);
	glRotatef(-90, 1, 0, 0);
	glRotatef(tilt, 1, 0, 0);
	glCallList(model_missile);
	glPopMatrix();

	/*glBegin(GL_TRIANGLES);
		glVertex3f(x, y, z);
		glVertex3f(x-1, y, z-1);
		glVertex3f(x-1, y, z+1);
	glEnd();*/
}

bool Projectile::isInside(float x, float y, float z) {
	if(x >= this->x+missile_minx
	&& x <= this->x+missile_maxx
	&& z >= this->z+missile_minz
	&& z <= this->z+missile_maxz
	&& y >= this->y+missile_miny
	&& y <= this->y+missile_maxy)
		return true;
	return false;
}
