/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Stars
#include <math.h>
#include "Star.h"
#include "main.h"
#include "gfx.h"

Star::Star() {
	// Place a random star in the top half of a sphere of radius 250
	// We do this by selecting a random x value.  From here we pick
	// a random sign for y, and make it less than the value it must be
	// to lie on a circle of radius 250.  From these values we
	// determine what the z must be.
	x = -250 + rand()%500000/1000.0;
	z = (rand()%2?-1:1)*rand()%int(sqrt(250*250-x*x));
	y = sqrt(250*250 - x*x - z*z);
	
	tex_idx = STAR00 + rand()%4;
	speed = 10 + rand()%20000/1000.0;
	counter = 0.0;
	
	radius = 0.75;
	shimmer = true;
}


void Star::Update(float dt) {
	counter += speed*dt;
	if(counter > 20.0) counter = 0.0;
}

void Star::Draw() {
	glPushMatrix();
	glTranslatef(x, y, z);
	glRotatef(180-camera.rotation, 0, 1, 0);
	glRotatef(180-camera.tilt, 0, 0, 1);
	

	glDepthMask(GL_FALSE);
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_FOG);
	glDisable(GL_LIGHTING);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glBindTexture(GL_TEXTURE_2D, texture[tex_idx]);
	glBegin(GL_QUADS);
		glColor4f(1.0, 1.0, 1.0, 1.0);
		glTexCoord2i(0,0);	glVertex3f(0, -radius, -radius);
		glTexCoord2i(0,1);	glVertex3f(0, -radius, radius);
		glTexCoord2i(1,1);	glVertex3f(0, radius, radius);
		glTexCoord2i(1,0);	glVertex3f(0, radius, -radius);
	glEnd();
	
	if(shimmer) {
		glBindTexture(GL_TEXTURE_2D, texture[TWINKLE]);
		glBegin(GL_QUADS);
			glColor4f(1.0, 1.0, 1.0, 0.1);
			float mult;
			if(counter < 10.0) mult = radius*4/3.0 + counter*4/15.0;
			else mult = radius*4 - (counter-10.0)*4/15.0;
			glTexCoord2i(0,0);	glVertex3f(0, -1*mult, -1*mult);
			glTexCoord2i(0,1);	glVertex3f(0, -1*mult, 1*mult);
			glTexCoord2i(1,1);	glVertex3f(0, 1*mult, 1*mult);
			glTexCoord2i(1,0);	glVertex3f(0, 1*mult, -1*mult);
		glEnd();
	}
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	glDepthMask(GL_TRUE);
	glEnable(GL_LIGHTING);
	glEnable(GL_FOG);
	glPopMatrix();
}
