/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Wave interface
#include "Wave.h"
#include "main.h"

#include <string.h>

Wave::Wave(const char *filename) {
	fprintf(stderr, "Reading wave file %s.\n", filename);
	FILE *f = fopen(filename, "rt");
	if(!f) {
		stop = true;
		return;
	}
	
	float time = 0.0;
	int cmds = 0;
	
	char line[512];
	while(!feof(f)) {
		fgets(line, 512, f);
		if(line[0] == '#' || line[0]==' ' || line[0]=='\n' || line[0]=='\t') continue;
	
		// Process what we just read
		cmds++;
		char cmd_str[256];
		float cmd_val;
		sscanf(line, "%s %f\n", cmd_str, &cmd_val);
		WaveBit *bit = new WaveBit();
		bit->value = cmd_val;
		if(strcasecmp(cmd_str, "wait")==0) {
			bit->cmd = WAVE_CMD_WAIT;
			time += cmd_val;
		}
		else if(strcasecmp(cmd_str,"plane")==0) bit->cmd = WAVE_CMD_PLANE;
		else {
			fprintf(stderr, "Unknown command in wave file: %s\n", cmd_str);
			delete bit;
			continue;
		}
		
		command.push(bit);
	}
	
	fclose(f);
	timer = next_timer = 0.0;
	stop = false;

	fprintf(stderr, "%d commands, requires %.0f seconds.\n", cmds, time);
}

Wave::~Wave() {
}


bool Wave::Update(float dt) {
	timer += dt;
	
	if(stop) return true;
	if(command.size() == 0) return false;
	
	if(timer > next_timer) {
		WaveBit *bit = command.front();
		command.pop();
		
		switch(bit->cmd) {
			case WAVE_CMD_WAIT:
				next_timer = timer + bit->value;
				printf("Waiting for %f seconds...\n", bit->value);
				break;
			
			case WAVE_CMD_PLANE:
			{
				int i;
				for(i=0; i<bit->value; i++) {
					Enemy *e = new Enemy();
					enemy.push_back(e);
				}
				printf("Spawning %0.0f airplanes.\n", bit->value);
			}
		}
	}
	
	
	return true;
}

