/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Drawing-related code
#include <alleggl.h>
#include <math.h>
#include <allegro.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "main.h"
#include "Explosion.h"
#include "Star.h"
#include "Building.h"
#include "Music.h"
#include "MyTrig.h"
#include "GlCompat.h"
#include "fudgefont.h"

const unsigned char COLOR_RADAR_PLANE[4] = { 255, 0, 0, 255 };
const unsigned char COLOR_RADAR_MISSILE[4] = {0, 0, 255, 255 };



void draw_ground() {
	static bool first = true;
	static GLuint number;
	
	glDisable(GL_LIGHTING);
	if(first) {
		int i, j;
		first = false;
		number = glGenLists(1);
				
		glNewList(number, GL_COMPILE_AND_EXECUTE);
		glBegin(GL_QUADS);
		for(i=-240; i<=240; i+=10) for(j=-240; j<=240; j+=10) {
			glNormal3f(0, 1, 0);
			glColor3ub(24-rand()%24, 75, 24-rand()%24);	glVertex3f(i, 0, j);
			glColor3ub(24-rand()%24, 75, 24-rand()%24);	glVertex3f(i, 0, j+10);
			glColor3ub(24-rand()%24, 75, 24-rand()%24);	glVertex3f(i+10, 0, j+10);
			glColor3ub(24-rand()%24, 75, 24-rand()%24);	glVertex3f(i+10, 0, j);		
		}
		glEnd();
		glEndList();
	}
	else {
		glCallList(number);
	}
	glEnable(GL_LIGHTING);
}



void DrawGameover(float dt) {
	static int select_color = makeacol(118,31,31,192);
	
	allegro_gl_set_allegro_mode();
		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glBegin(GL_QUADS);
			if(easygame)
				gl_rectfill(screen, SCREEN_W/2-100, SCREEN_H/2-10, SCREEN_W/2+100, SCREEN_H/2+25, select_color);
			else
				gl_rectfill(screen, SCREEN_W/2-100, SCREEN_H/2+20, SCREEN_W/2+100, SCREEN_H/2+55, select_color);
		glEnd();
	
		glColor4f(1,1,1, 0.5);
		my_print(menu_font_huge, SCREEN_W/2, 48, TRUE, "Raid 2");
		my_print(menu_font_normal, SCREEN_W/2, SCREEN_H/2-10, TRUE, "Play easy game");
		my_print(menu_font_normal, SCREEN_W/2, SCREEN_H/2+20, TRUE, "Play hard game");
	allegro_gl_unset_allegro_mode();
	
}


void DrawGame(float dt) {
	allegro_gl_set_allegro_mode();
		glColor3f(0.8, 0.1, 0.1);
		if(current_wave <= 10)
			my_print(menu_font_normal, SCREEN_W/2, 10, TRUE, "WAVE %d", current_wave);
		else {
			my_print(menu_font_huge, SCREEN_W/2, SCREEN_H/2-40, TRUE, "CONGRATULATIONS!");
			my_print(menu_font_normal, SCREEN_W/2, SCREEN_H/2+40, TRUE, "You have destroyed all of Saddam Hussein's");
			my_print(menu_font_normal, SCREEN_W/2, SCREEN_H/2+65, TRUE, "evil forces which were bent on world domination.");
			my_print(menu_font_normal, SCREEN_W/2, SCREEN_H/2+100, TRUE, "Thank you for playing!");
		}
	allegro_gl_unset_allegro_mode();
}


void DrawAlways(float dt) {
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity();
	
	
	glLoadIdentity();
	gluPerspective(66.66667, SCREEN_W/(float)SCREEN_H, 1.0, 1000.0);

	glRotatef(90, 0, 1, 0);	// Tilt so that y points up
	glRotatef(-camera.tilt, 0, 0, 1);
	glRotatef(camera.rotation, 0, 1, 0);
	glTranslatef(0,-camera.z,0);	// Go up z meters in the sky
	
	// The stars
	for(list<Star*>::iterator it=star.begin(); it!=star.end(); ++it) (*it)->Draw();

	// The enemies
	for(list<Enemy*>::iterator it=enemy.begin(); it!=enemy.end(); ++it) (*it)->Draw();
	
	// The projectiles
	for(list<Projectile*>::iterator it=projectile.begin(); it!=projectile.end(); ++it) (*it)->Draw();
	
	// The buildings
	for(vector<Building*>::iterator it=building.begin(); it!=building.end(); ++it) (*it)->Draw();	

	// The ground
	draw_ground();

	// The explosions
	for(list<Explosion*>::iterator it=explosion.begin(); it!=explosion.end(); ++it) (*it)->Draw();
	

	
	// Interface stuff
	//glViewport(0, 0, SCREEN_W, SCREEN_H);
	allegro_gl_set_allegro_mode();
		glColor3f(1,1,1);
		my_print(menu_font_normal, 5, 0, FALSE, "Score: %d", camera.score);
		//my_print_right(menu_font_normal, SCREEN_W-5, 0, "fps: %0.0f", 1.0/dt);
		my_print_right(menu_font_normal, SCREEN_W-5, 0, "%d Buildings Left", camera.buildings_left);
	allegro_gl_unset_allegro_mode();
	
	// This unset and reset is unfortunately necessary at the moment
	allegro_gl_set_allegro_mode();		
		// Status of music player
		textprintf_centre_ex(screen, font, SCREEN_W/2, SCREEN_H-10, makecol(255,255,255), -1, "%s", music->StatusString());

		glEnable(GL_BLEND);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		// The radar
		if(radar_on) {
			
			glPushMatrix();
			glTranslatef(SCREEN_W/2, SCREEN_H/2, 0);
			glRotatef(90-camera.rotation, 0, 0, 1);
			glScalef(-1, -1, 1);
			
			// Use OpenGL to make a translucent circle
			static GLuint circle_idx=0;
			if(!circle_idx) {
				int i;
				circle_idx = glGenLists(1);
				glNewList(circle_idx, GL_COMPILE);
				glBegin(GL_POLYGON);
				const static int points=60;
				glColor4ub(49, 119, 21, 64);
				for(i=0; i<points; i++) {
					glVertex2f(cos(i/(float)points*2*M_PI), sin(i/(float)points*2*M_PI));
				}
				glEnd();
				glEndList();
			}
			
			glPushMatrix();
			glScalef(radar_radius, radar_radius, 1);
			glCallList(circle_idx);
			glPopMatrix();
			
			
			// Make display list for small circles
			static GLuint point_idx=0;
			if(!point_idx) {
				point_idx = glGenLists(1);
				glNewList(point_idx, GL_COMPILE);
				
				glBegin(GL_POLYGON);
					int i;
					const static int points = 8;
					for(i=0; i<points; i++)
						glVertex2f(cos(i/(float)points*2*M_PI), sin(i/(float)points*2*M_PI));
				glEnd();
				glEndList();
			}
			
			// Draw all projectiles on radar
			glColor4ubv(COLOR_RADAR_MISSILE);
			for(list<Projectile*>::iterator it=projectile.begin(); it!=projectile.end(); ++it) {
				Projectile *p = *it;
			
				if(p->x*p->x + p->z*p->z <= FIELD_W*FIELD_L/4) {
					glPushMatrix();
					glTranslatef(p->x/FIELD_W*radar_radius*2, p->z/FIELD_W*radar_radius*2,0);
					glScalef(radar_radius/100.0, radar_radius/100.0, 1.0);
					glCallList(point_idx);
					glPopMatrix();
				}
			}
				

			// Draw all enemies on radar
			glColor4ubv(COLOR_RADAR_PLANE);
			for(list<Enemy*>::iterator it=enemy.begin(); it!=enemy.end(); ++it) {
				Enemy *e = *it;
				
				// If the enemy is in one of the corners, do not draw it
				if(e->x*e->x + e->z*e->z <= FIELD_W*FIELD_L/4) {					
					// Draw the blip on the radar
					glPushMatrix();
					glTranslatef(e->x/FIELD_W*radar_radius*2, e->z/FIELD_L*radar_radius*2, 0);
					glScalef(radar_radius/100.0, radar_radius/100.0, 1.0);
					glCallList(point_idx);
					glPopMatrix();
				}
				
			}
			glEnd();
			glPopMatrix();
			hline(screen, -15, 0, 15, 0);
			line(screen, 15, 0, 5, 10, 0);
			line(screen, 15, 0, 5, -10, 0);
		}
		
		// Crosshairs
		glEnable(GL_BLEND);
		glBegin(GL_QUADS);
			glColor4ub(50, 50, 50, 127);
		
			glVertex2f(SCREEN_W/2 - 4, SCREEN_H/2 - 48);
			glVertex2f(SCREEN_W/2 - 4, SCREEN_H/2 - 8);
			glVertex2f(SCREEN_W/2 + 4, SCREEN_H/2 - 8);
			glVertex2f(SCREEN_W/2 + 4, SCREEN_H/2 - 48);
		
			glVertex2f(SCREEN_W/2 - 4, SCREEN_H/2 + 8);
			glVertex2f(SCREEN_W/2 - 4, SCREEN_H/2 + 48);
			glVertex2f(SCREEN_W/2 + 4, SCREEN_H/2 + 48);
			glVertex2f(SCREEN_W/2 + 4, SCREEN_H/2 + 8);
		
			glVertex2f(SCREEN_W/2 - 48, SCREEN_H/2 - 4);
			glVertex2f(SCREEN_W/2 - 48, SCREEN_H/2 + 4);
			glVertex2f(SCREEN_W/2 - 8, SCREEN_H/2 + 4);
			glVertex2f(SCREEN_W/2 - 8, SCREEN_H/2 - 4);
		
			glVertex2f(SCREEN_W/2 + 8, SCREEN_H/2 - 4);
			glVertex2f(SCREEN_W/2 + 8, SCREEN_H/2 + 4);
			glVertex2f(SCREEN_W/2 + 48, SCREEN_H/2 + 4);
			glVertex2f(SCREEN_W/2 + 48, SCREEN_H/2 - 4);
		glEnd();
		glDisable(GL_BLEND);
		
		
	allegro_gl_unset_allegro_mode();
}
