/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Input-related code
#include <allegro.h>
#include "main.h"
#include "sfx.h"
#include "Music.h"


void KeycheckGameover(float dt) { 
	if(key[KEY_UP]) easygame = true;
	if(key[KEY_DOWN]) easygame = false;
	
	if(key[KEY_ENTER] || key[KEY_ENTER_PAD]) {
		gameover=false;
		if(firstgame) firstgame = false;
		else reset_game();
	}
}

void KeycheckGame(float dt) {
	static float last_shot = 100.0;
	last_shot += dt;
	if((key[KEY_SPACE] || (mouse_b & 1)) && last_shot >= 0.1) {
		last_shot = 0.0;
		Projectile *p = new Projectile(0, camera.z, 0, camera.rotation, camera.tilt, 100);
		projectile.push_back(p);
		
		camera.score += SCORE_SHOOT;
		if(camera.score < 0) camera.score = 0;
		camera.missiles_fired++;
		play_sample((SAMPLE*)sfx[SFX_SHOOT].dat, 255, 127, 1000, FALSE);
	}
}


void KeycheckAlways(float dt) {
	if(key[KEY_LEFT]) {
		camera.rotation -= 100*dt;
		if(camera.rotation < 0) camera.rotation += 360;
	}
	
	if(key[KEY_RIGHT]) {
		camera.rotation += 100*dt;
		if(camera.rotation > 360) camera.rotation -= 360;
	}
	
	if(key[KEY_PLUS_PAD] || key[KEY_EQUALS]) {
		radar_radius += MAX(SCREEN_W, SCREEN_H)/100.0;
		if(radar_radius >= MIN(SCREEN_W, SCREEN_H)/2) radar_radius = MIN(SCREEN_W, SCREEN_H)/2;
	}
	
	if(key[KEY_MINUS_PAD] || key[KEY_MINUS]) {
		radar_radius -= MAX(SCREEN_W, SCREEN_H)/100.0;
		if(radar_radius <= 10.0) radar_radius = 10.0;
	}
	
	
	if(keypressed()) {
		int k = readkey() >> 8;
		switch(k) {
			// Toggle the radar
			case KEY_F2: radar_on = !radar_on; break;
			
			// New song
			case KEY_F5: music->PlayRandomSong(); break;
			
			// Take a screenshot
			case KEY_F12:
			{
				// Determine the next unused screenshot filename
				static int next_idx=0;
				static int first=true;
				char fn[32];
				if(first) {
					first = false;
					while(1) {
						snprintf(fn, 32, "r2-%05d.bmp", next_idx);
						if(!file_exists(fn, 0, NULL)) break;
						++next_idx;
					}
				}
				
				
				BITMAP *b = create_bitmap(SCREEN_W, SCREEN_H);
				blit(screen, b, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
				snprintf(fn, 32, "r2-%05d.bmp", next_idx);
				save_bmp(fn, b, NULL);
				++next_idx;
				break;
			}
		}
	}
	
	
	// Also, mouse input
	int mx, my;
	float oldrot = camera.rotation;
	float oldtilt = camera.tilt;
	get_mouse_mickeys(&mx, &my);
	camera.rotation += 5*mx*dt;
	camera.tilt += 5*my*dt;
	
	if(key[KEY_DOWN]) camera.tilt += 40*dt;
	if(key[KEY_UP]) camera.tilt -= 40*dt;
	
	// Make sure that we don't rotate beyond our capabilities
	if(camera.tilt <= -40.0) camera.tilt = -40.0;
	else if(camera.tilt >= 220.0) camera.tilt = 220.0;
}
