/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#include <math.h>
#include <alleggl.h>
#include <GL/glu.h>
#include <GL/glut.h>
#include <allegro.h>
#include <alogg/alogg.h>
#include <list>
#include <string.h>

#include <time.h>

#include <curl/curl.h>

#include "MyTrig.h"
#include "Timer.h"
#include "Enemy.h"
#include "Projectile.h"
#include "Building.h"
#include "Explosion.h"
#include "Star.h"
#include "Wave.h"
#include "Model.h"
#include "Music.h"
#include "gfx.h"
#include "sfx.h"

#include "fudgefont.h"

#define IN_MAIN_CC
#include "main.h"



int fetch_progress_callback(void *data, double dltotal, double dlnow, double ultotal, double ulnow) {
	//printf("Progress: %f%%\n", dlnow/dltotal*100);
	return 0;
}


bool fetch_from_net(const char *url) {
	bool first = true;
	CURL *handle;
	char errstr[1024];
	
	if(first) {
		// Initialize cURL
		curl_global_init(CURL_GLOBAL_ALL);
	}
	
	// Determine the filename from the URL
	char *filename = rindex(url, '/');
	if(filename == NULL) return false;
	filename++;
	
	// Open the file for output
	FILE *f = fopen(filename, "wb");
	if(!f) return false;
	
	// Set cURL options
	handle = curl_easy_init();
	curl_easy_setopt(handle, CURLOPT_WRITEDATA, f);
	curl_easy_setopt(handle, CURLOPT_NOPROGRESS, 0);
	curl_easy_setopt(handle, CURLOPT_FOLLOWLOCATION, 1);
	curl_easy_setopt(handle, CURLOPT_PROGRESSFUNCTION, &fetch_progress_callback);
	curl_easy_setopt(handle, CURLOPT_PROGRESSDATA, NULL);
	curl_easy_setopt(handle, CURLOPT_ERRORBUFFER, errstr);
	
	// Download the file
	curl_easy_setopt(handle, CURLOPT_URL, url);
	
	if(curl_easy_perform(handle) != 0) return false;
	curl_easy_cleanup(handle);
	fclose(f);
	return true;	
}



void create_buildings() {
	camera.buildings_left = 0;
	
	
	int i, j;
	// Create some random-height buildings for us to enjoy
	camera.buildings_left = 0;
	for(i=-155; i<=150; i+=30) for(j=-155; j<=150; j+=30) {
		// Look at how lazy I am!
		if(i*i+j*j>=150*150)continue;
		Building *b = new Building(i, j, 10 + rand()%4000/100.0);
		if(abs(i) < 10 && abs(j) < 10) {
			b->height = 48;
			main_building = building.size();
		}
		building.push_back(b);
		alive_building.push_back(b);
		camera.buildings_left++;
	}
}



void init() {
	// Some various values
	radar_radius = MAX(SCREEN_H, SCREEN_W)/5.0;
	radar_on = true;	
	
	camera.rotation = camera.tilt = 0.0;
	camera.z = 50.0;	

	// Some OpenGL stuff
	float clear_color[] = { 0.0234375/4,0.0859375/4,0.2109375/4,0.0 };
	glClearColor(clear_color[0],clear_color[1],clear_color[2],clear_color[3]);
	glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POINT_SMOOTH_HINT, GL_NICEST);
	glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);
	glPointSize(3.0);
	
	glFogi(GL_FOG_MODE, GL_LINEAR);
	glFogi(GL_FOG_START, 75);
	glFogi(GL_FOG_END, 190);
	//float black[] = {0, 0, 0, 0};
	glFogfv(GL_FOG_COLOR, clear_color);
	glEnable(GL_FOG);
	
	glShadeModel(GL_SMOOTH);
	
	// Lighting stuff
	float light_pos[3] = { 0, camera.z, 0 };
	float mat_specular[] = { 0.5, 0.5, 1, 1 };
	float mat_shine[] = { 50 };
	glMaterialfv(GL_FRONT, GL_SPECULAR, mat_specular);
	glMaterialfv(GL_FRONT, GL_SHININESS, mat_shine);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glEnable(GL_LIGHTING);
	glEnable(GL_LIGHT0);
	
	
	// Backface culling
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);
	glEnable(GL_CULL_FACE);
	

	int i;
	create_buildings();

	// Create the stars, randomly
	for(i=0; i<250; i++) {
		Star *s = new Star();
		star.push_back(s);
	}
	
	// And hardcode the moon, friggin huge :)
	Star *s = new Star();
	s->radius = 50.0;
	s->tex_idx = THEMOON;
	s->shimmer = false;
	s->x = 50.0;
	s->y = 150.0;
	s->z = sqrt(250*250 - 50*50-150*150);
	star.push_back(s);
	
	
	// Load the models
	model_airplane = LoadModel("airplane.raw", &plane_minx, &plane_maxx, &plane_miny, &plane_maxy, &plane_minz, &plane_maxz);
	model_missile = LoadModel("missile.raw", &missile_minx, &missile_maxx, &missile_miny, &missile_maxy, &missile_minz, &missile_maxz);
	

	// Texture stuff
	float all_alpha[] = { 0, 0, 0, 1 };
	allegro_gl_use_mipmapping(TRUE);
	allegro_gl_use_alpha_channel(TRUE);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
	glTexParameterfv(GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, all_alpha);

	// Load textures
	char gfx_file[512]="gfx.dat";
	/*if(!file_exists(gfx_file, 0, NULL)) {
		int choice = algl_alert("You do not have the extra data package.","Would you like to download","additional art and graphics?","&Yes, please!", "&No thank you", KEY_Y, KEY_N);
		if(choice == 1) {
			fetch_from_net("http://members.allegro.cc/stevenvi/raid2/gfx.dat");
		}
		else snprintf(gfx_file, 512, "gfx-lo.dat");
	}*/
	gfx = load_datafile(gfx_file);
	if(!gfx) {
		set_gfx_mode(GFX_TEXT,0,0,0,0);
		allegro_message("Error loading %s.\n", gfx_file);
		exit(1);
	}

	fprintf(stderr, "Processing textures in gfx.dat...");
	for(i=0; gfx[i].type == DAT_BITMAP; i++) {
		texture[i] = allegro_gl_make_texture((BITMAP*)gfx[i].dat);
	}
	fprintf(stderr, "...%d textures processed.\n", i);
	
	
	// Load fonts using FudgeFont
	install_fudgefont();
	if(!file_exists("HoboStd.ttf", 0, NULL)) {
		set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
		allegro_message("Font file HoboStd.ttd not found.\n");
		exit(1);
	}
	menu_font_huge = my_load_font("HoboStd.ttf", 72);
	menu_font_normal = my_load_font("HoboStd.ttf", 24);
	
	// Load SFX
	sfx = load_datafile("sfx.dat");


	music->PlayFirstSong();
	wave = new Wave("wave01.txt");
	
	
	// Start out the game in a game over state
	gameover=true;
	easygame=true;
	current_wave = 1;
	firstgame = true;
}


void reset_game() {
	// Start a new song playing
	music->PlayRandomSong();
	
	current_wave = 1;
	gameover = false;
	
	// Destroy everything
	building.clear();
	alive_building.clear();
	enemy.clear();
	projectile.clear();
	explosion.clear();
	
	create_buildings();
}


Timer timer;


void scan_for_errors() {
	// OpenGL errors
	/*while(1) {
		GLubyte err = glGetError();
		if(err == GL_NO_ERROR) break;
		fprintf(stderr, "OpenGL Error %d: %s\n", err, gluErrorString(err));
	}*/
}



int main(int argc, char *argv[]) {
	// Begin initialization
	srand(time(0));
        allegro_init();
        install_allegro_gl();
	glutInit(&argc, argv);
	
	set_window_title("Raid 2: Saddam Hussein's Revenge");
        allegro_gl_set(AGL_Z_DEPTH, 8);
        allegro_gl_set(AGL_COLOR_DEPTH, 24);
        allegro_gl_set(AGL_SUGGEST, AGL_Z_DEPTH | AGL_COLOR_DEPTH);

        if(set_gfx_mode(GFX_OPENGL_WINDOWED, 800, 600, 0, 0)) {
                set_gfx_mode(GFX_TEXT, 0, 0, 0, 0);
                allegro_message("Error setting gfx mode.\n%s\n", allegro_error);
                return 1;
        }

	install_keyboard();
	install_mouse();
	install_sound(DIGI_AUTODETECT, MIDI_NONE, NULL);
	set_volume_per_voice(0);
	alogg_init();
	
	// Load the playlist, start downloading music
	music = new Music("playlist.txt");

        glMatrixMode(GL_MODELVIEW);
	timer.timeElapsed();
	glEnable(GL_DEPTH_TEST);
	
	init();
	
	
	// Start the game loop
	while(!key[KEY_ESC]) {
		float dt = timer.timeElapsed();
		if(key[KEY_LSHIFT]) dt *= 10;
		
		scan_for_errors();
		
		UpdateAlways(dt);
		KeycheckAlways(dt);
		DrawAlways(dt);

		if(!gameover) {
			KeycheckGame(dt);
			UpdateGame(dt);
			DrawGame(dt);
		}
		else {
			KeycheckGameover(dt);
			UpdateGameover(dt);
			DrawGameover(dt);
		}
		
		allegro_gl_flip();
		rest(1);
	}

	alogg_exit();
        return 0;
}
END_OF_MAIN()

