/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

#ifndef _MAIN_H
#define _MAIN_H

#ifdef IN_MAIN_CC
#define EXTERN
#else
#define EXTERN extern
#endif

#include <GL/gl.h>
#include <list>
#include <vector>
#include "Building.h"
#include "Enemy.h"
#include "Projectile.h"

using namespace std;

#define SCORE_PLANE	1000
#define SCORE_MISSILE	5000
#define SCORE_BUILDING	-1000
#define SCORE_SHOOT	-10

#define FIELD_W	300.0
#define FIELD_L	300.0


EXTERN vector<class Building*> building;
EXTERN list<class Building*> alive_building;
EXTERN list<class Enemy*> enemy;
EXTERN list<class Projectile*> projectile;
EXTERN list<class Explosion*> explosion;
EXTERN list<class Star*> star;

EXTERN float radar_radius;			// Radius of in-game radar
EXTERN bool radar_on;				// On/off state of in-game radar
EXTERN bool gameover;				// If we are in a game over state
EXTERN bool easygame;				// If we are playing an easy game
EXTERN bool firstgame;				// If we haven't yet played a game
EXTERN int main_building;			// index in vector<Building*> building which holds the main building
EXTERN int current_wave;			// current wave we are in
EXTERN class Music *music;
EXTERN class Wave *wave;
EXTERN struct FONT *menu_font_huge, *menu_font_normal;


EXTERN GLuint model_airplane, model_missile;
EXTERN float plane_minx, plane_maxx, plane_miny, plane_maxy, plane_minz, plane_maxz;
EXTERN float missile_minx, missile_maxx, missile_miny, missile_maxy, missile_minz, missile_maxz;

// Graphics
EXTERN struct DATAFILE *gfx;
EXTERN GLuint texture[32];

// Sound Effects
EXTERN struct DATAFILE *sfx;



void UpdateAlways(float dt);
void UpdateGame(float dt);
void UpdateGameover(float dt);

void KeycheckAlways(float dt);
void KeycheckGame(float dt);
void KeycheckGameover(float dt);

void DrawAlways(float dt);
void DrawGame(float dt);
void DrawGameover(float dt);


void reset_game();


typedef struct Vector {
	float x, y, z;
};


EXTERN struct {
	float rotation;
	float tilt;
	float z;
	
	int buildings_left;
	int planes_destroyed;
	int missiles_destroyed;
	int missiles_fired;
	int score;
} camera;


#endif // _MAIN_H
