/*
 * Raid 2: The Return of Saddam
 * Copyright (c) 2008 Steven Wallace / Chedda Cheeze
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 2 of the License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA
 */

// Update-related code
#include "main.h"
#include <allegro.h>
#include "Explosion.h"
#include "MyTrig.h"
#include "sfx.h"
#include "Wave.h"
#include <math.h>
#include "Star.h"
#include "Music.h"
#include "Enemy.h"
#include "Projectile.h"
#include "Building.h"
#include <string.h>


void play_boom(float x, float y, float z) {
	// Determine distance for volume
	float dist = sqrt(x*x+(y-camera.z)*(y-camera.z)+z*z);
	
	// Determine angle for pan
	float angle = camera.rotation - atan2(z,x)*TODEG;
	if(angle < 270 && angle > 90) angle = 180 - angle;
	
	play_sample((SAMPLE*)sfx[SFX_BOOM00+rand()%4].dat, int(256*dist/150), 127 + int(angle/90*128), 850 + rand()%301, FALSE);
}


void UpdateGameover(float dt) {
}


void UpdateGame(float dt) {
	if(!wave->Update(dt)) {
		// Go to the next wave
		char fn[32];
		snprintf(fn, 32, "wave%02d.txt", ++current_wave);
		delete wave;
		wave = new Wave(fn);
	}
	
	// If the turret building goes down or all other buildings do, game is over
	if(camera.buildings_left == 1 || building[main_building]->alive == false) {
		gameover=true;
		music->PlayLastSong();
	}
}


void UpdateAlways(float dt) {
	music->PollMusic(dt);


	// Update the stars
	for(list<Star*>::iterator it=star.begin(); it!=star.end(); ++it) (*it)->Update(dt);
	
	// Update all explosion stuff
	for(list<Explosion*>::iterator it=explosion.begin(); it!=explosion.end(); ++it) {
		Explosion *e = *it;
		if(!e->Update(dt)) {
			it = explosion.erase(it);
			delete e;
			continue;
		}
	}
	
	// Update all enemies
	for(list<Enemy*>::iterator it=enemy.begin(); it!=enemy.end(); ++it) {
		Enemy *e = *it;
		if(!e->Update(dt)) {
			it = enemy.erase(it);
			delete e;
			continue;
		}
	}
	
	// Update all projectiles
	PROJECTILE_ITERATION:
	for(list<Projectile*>::iterator it=projectile.begin(); it!=projectile.end(); ++it) {
		Projectile *p = *it;
		if(!p->Update(dt)) {
			it = projectile.erase(it);
			delete p;
			continue;
		}
		else {
			// See if it hit something
			// First check the buildings
			for(list<Building*>::iterator itb=alive_building.begin(); itb!=alive_building.end(); ++itb) {
				Building *b = *itb;
				if(b->alive && b->isInside(p->x, p->y, p->z)) {
					// Hit!
					alive_building.remove(b);
					play_boom(p->x, p->y, p->z);
					int i;
					for(i=4; i<ceil(b->height); i+=4) {
						int j, jmax = 5 + rand()%16;
						for(j=0; j<jmax; j++) {
							Explosion *e = new Explosion(b->x+rand()%1000/100.0, i+rand()%100/100.0, b->y+rand()%1000/100.0, EX_TYPE_FIRE);
							explosion.push_back(e);
						}
						
						jmax = 2 + rand()%6;
						for(j=0; j<jmax; j++) {
							Explosion *e = new Explosion(b->x+rand()%1000/100.0, i+rand()%100/100.0, b->y+rand()%1000/100.0, EX_TYPE_SMOKE);
							explosion.push_back(e);
						}
					}
					camera.score += SCORE_BUILDING;
					if(camera.score < 0) camera.score = 0;
					camera.buildings_left--;
					b->alive = false;
					b->height = 4;
					delete p;
					it = projectile.erase(it);
					goto PROJECTILE_ITERATION;
				}
			}
			
			// Now check the planes
			for(list<Enemy*>::iterator ite=enemy.begin(); ite!=enemy.end(); ++ite) {
				Enemy *e = *ite;
				
				// Make sure that we don't blow up the plane dropping us!
				float dy = p->start_y - p->y;
				if(dy > 0.0 && dy < 15.0) continue;
				
				if(e->isInside(p->x, p->y, p->z)) {
					// Hit!
					play_boom(p->x, p->y, p->z);
					int j, jmax = 20 + rand()%10;
					for(j=0; j<jmax; j++) {
						Explosion *e2 = new Explosion(e->x+rand()%1000/100.0, e->y+rand()%1000/100.0, e->z+rand()%1000/100.0, EX_TYPE_FIRE);
						explosion.push_back(e2);
					}
					jmax = j/2;
					for(j=0; j<jmax; j++) {
						Explosion *e2 = new Explosion(e->x+rand()%1000/100.0, e->y+rand()%1000/100.0, e->z+rand()%1000/100.0, EX_TYPE_SMOKE);
						explosion.push_back(e2);
					}
					camera.score += SCORE_PLANE;
					camera.planes_destroyed++;
					delete e;
					delete p;
					ite = enemy.erase(ite);
					it = projectile.erase(it);
					goto PROJECTILE_ITERATION;
				}
			}
			
			// Check for other projectiles
			for(list<Projectile*>::iterator itp = it; ;) {
				++itp; if(itp==projectile.end()) break;
				Projectile *p2 = *itp;
				if(p2->isInside(p->x, p->y, p->z)) {
					// Two projectiles nullify each other
					play_boom(p->x, p->y, p->z);
					int j, jmax = 10+rand()%10;
					for(j=0; j<jmax; j++) {
						Explosion *e = new Explosion(p2->x+rand()%500/100.0, p2->y + rand()%500/100.0, p2->z+rand()%500/100.0, EX_TYPE_FIRE);
						explosion.push_back(e);
					}
					camera.score += SCORE_MISSILE;
					camera.missiles_destroyed++;
					delete p2;
					delete p;
					itp = projectile.erase(itp);
					it = projectile.erase(it);
					goto PROJECTILE_ITERATION;
				}
			}
		}
	}
}
